//  Alice.swift
//  ALiCE Biometrics
//  Copyright © 2020+ ALiCE Biometrics. All rights reserved.


import Foundation
import AliceOnboarding
import AVFoundation

@objc(Alice)
class Alice: NSObject {
    
    @objc(runOnboarding:config:resolve:reject:)
    func runOnboarding(userToken:String,
                       config:NSDictionary,
                       resolve: @escaping RCTPromiseResolveBlock,
                       reject: @escaping RCTPromiseRejectBlock) {

        let configDictionary = getConfigDictionary(nsDictionaryConfig: config)

        AVCaptureDevice.requestAccess(for: AVMediaType.video) { response in
            if response {
                DispatchQueue.main.async { () -> Void in
                    let rootViewController = UIApplication.shared.windows.first!.rootViewController!
                    onboardingBindingFromDictionary(
                        rootViewController,
                        userToken: userToken,
                        onboardingConfigDictionary: configDictionary
                    ){ result in
                        switch result {
                        case let .success(userStatus):
                            resolve(String(describing: userStatus))
                        case let .failure(error):
                            reject("error", stringFromOnboardingError(error), error)
                        case .cancel:
                            reject("error", "cancel", nil)
                        case .dismissButton:
                            reject("error", "dismissButton", nil)
                        @unknown default:
                            reject("error", "unknown", nil)
                        }
                    }
                }
            } else {
                reject("error", "camera permissions not granted", nil)
            }
        }
    }
    
    @objc(commandGetUserStatus:resolve:reject:)
      func commandGetUserStatus(userToken:String,
                         resolve: @escaping RCTPromiseResolveBlock,
                         reject: @escaping RCTPromiseRejectBlock) {
        
       DispatchQueue.main.async { () -> Void in
        let rootViewController = UIApplication.shared.windows.first!.rootViewController!
        let appearanceConfig: AppearanceConfig = AppearanceConfig()
        
        let onboardingCommands = OnboardingCommands(rootViewController, userToken: userToken,
                                                     appearanceConfig: appearanceConfig
                                                    ) { error in
                                                        
                                                      reject("error", stringFromOnboardingError(error), error)
        }
        
        onboardingCommands.getUserStatus { result in
                 switch result {
                 case .success(let onboardingResponse):
                    resolve(onboardingResponse.content ?? "")
                 case let .failure(error):
                    reject("error", stringFromOnboardingError(error), error)
                 case .cancel:
                    reject("error", "cancel", nil)
                 case .dismissButton:
                    reject("error", "dismissButton", nil)
                 @unknown default:
                    reject("error", "unknown", nil)
                 }
             }
          }
      }
    
    @objc(commandGetDocumentsSupported:resolve:reject:)
       func commandGetDocumentsSupported(userToken:String,
                          resolve: @escaping RCTPromiseResolveBlock,
                          reject: @escaping RCTPromiseRejectBlock) {
         
        DispatchQueue.main.async { () -> Void in
         let rootViewController = UIApplication.shared.windows.first!.rootViewController!
         let appearanceConfig: AppearanceConfig = AppearanceConfig()
         
         let onboardingCommands = OnboardingCommands(rootViewController, userToken: userToken,
                                                     appearanceConfig: appearanceConfig) { error in
                                                         
                                                       reject("error", stringFromOnboardingError(error), error)
         }
         
            onboardingCommands.getDocumentsSupported { result in
                  switch result {
                  case .success(let onboardingResponse):
                     resolve(onboardingResponse.content ?? "")
                  case .failure(let error):
                       reject("error", stringFromOnboardingError(error), error)
                  case .cancel:
                     reject("error", "cancel", nil)
                  case .dismissButton:
                      reject("error", "dismissButton", nil)
                  @unknown default:
                      reject("error", "unknown", nil)
                }
              }
           }
       }
    
    
    @objc(commandAddSelfie:resolve:reject:)
    func commandAddSelfie(userToken:String,
                          resolve: @escaping RCTPromiseResolveBlock,
                          reject: @escaping RCTPromiseRejectBlock) {
        
        AVCaptureDevice.requestAccess(for: AVMediaType.video) { response in
                     if response {
                   DispatchQueue.main.async { () -> Void in
                       let rootViewController = UIApplication.shared.windows.first!.rootViewController!
                       let appearanceConfig: AppearanceConfig = AppearanceConfig()
                       let onboardingCommands = OnboardingCommands(rootViewController, userToken: userToken,
                                                                    appearanceConfig: appearanceConfig) { error in
                                                                                                                    
                                                                            reject("error", stringFromOnboardingError(error), error)
                                                                    }
                    
                         onboardingCommands.addSelfie { result in
                                  switch result {
                                  case let .success(response):
                                    resolve(response.content)
                                  case .failure(let error):
                                       reject("error", stringFromOnboardingError(error), error)
                                  case .cancel:
                                       reject("error", "cancel", nil)
                                  case .dismissButton:
                                       reject("error", "dismissButton", nil)
                                  @unknown default:
                                       reject("error", "unknown", nil)
                            }
                              }
                       }
                     } else {
                         reject("error", "camera permissions not granted", nil)
                   }
             }
    }
    
    @objc(commandCreateDocument:type:issuingCountry:resolve:reject:)
    func commandCreateDocument(userToken:String,
                               type :String,
                               issuingCountry :String,
                               resolve: @escaping RCTPromiseResolveBlock,
                               reject: @escaping RCTPromiseRejectBlock) {
        
        DispatchQueue.main.async { () -> Void in
           let rootViewController = UIApplication.shared.windows.first!.rootViewController!
           let appearanceConfig: AppearanceConfig = AppearanceConfig()
           let documentType = DocumentType.idcard
           let documentIssuingCountry = issuingCountry
           
           let onboardingCommands = OnboardingCommands(rootViewController, userToken: userToken,
                                                        appearanceConfig: appearanceConfig) { error in
                                                                                                        
                                                                    reject("error", stringFromOnboardingError(error), error)
                                                        }
        
            onboardingCommands.createDocument(
                type: documentType,
               issuingCountry: documentIssuingCountry) { resultCreateDocument in
                   switch resultCreateDocument {
                   case .success(let onboardingResponse):
                        resolve(onboardingResponse.content)
                   case .failure(let error):
                         reject("error", stringFromOnboardingError(error), error)
                   case .cancel:
                         reject("error", "cancel", nil)
                   case .dismissButton:
                         reject("error", "dismissButton", nil)
                   @unknown default:
                         reject("error", "unknown", nil)
                   }
               }
           }
        }
    
    @objc(commandAddDocument:documentId:type:side:issuingCountry:resolve:reject:)
    func commandAddDocument(userToken: String,
                            documentId: String,
                            type: String,
                            side:String,
                            issuingCountry: String,
                            resolve: @escaping RCTPromiseResolveBlock,
                            reject: @escaping RCTPromiseRejectBlock) {
        
        AVCaptureDevice.requestAccess(for: AVMediaType.video) { response in
               if response {
                   DispatchQueue.main.async { () -> Void in
                    let documentType = DocumentType.idcard
                    let documentSide = DocumentSide.front
                    let appearanceConfig: AppearanceConfig = AppearanceConfig()
                    let rootViewController = UIApplication.shared.windows.first!.rootViewController!
                
                    let onboardingCommands = OnboardingCommands(rootViewController, userToken: userToken,
                                                                   appearanceConfig: appearanceConfig) { error in
                                                                                                                   
                                                                            reject("error", stringFromOnboardingError(error), error)
                                                                   }
                       onboardingCommands.addDocument(
                                                    documentId: documentId,
                                                    type: documentType,
                                                    issuingCountry: issuingCountry,
                                                    side: documentSide) { resultAddDocument in
                                                        switch resultAddDocument {
                                                        case .success(let onboardingResponse):
                                                            resolve("Document added sucessfully")
                                                        case .failure(let error):
                                                            reject("error", stringFromOnboardingError(error), error)
                                                        case .cancel:
                                                             reject("error", "cancel", nil)
                                                        case .dismissButton:
                                                              reject("error", "dismissButton", nil)
                                                        @unknown default:
                                                              reject("error", "unknown", nil)
                                                        }
                                                }
                   }
               } else {
                   reject("error", "camera permissions not granted", nil)
               }
           }
    }
    
    @objc(commandAuthenticate:resolve:reject:)
          func commandAuthenticate(userToken:String,
                                   resolve: @escaping RCTPromiseResolveBlock,
                                   reject: @escaping RCTPromiseRejectBlock) {
            
        AVCaptureDevice.requestAccess(for: AVMediaType.video) { response in
                                 if response {
                               DispatchQueue.main.async { () -> Void in
                                   let rootViewController = UIApplication.shared.windows.first!.rootViewController!
                                   let appearanceConfig: AppearanceConfig = AppearanceConfig()
                                   let onboardingCommands = OnboardingCommands(rootViewController, userToken: userToken,
                                                                                appearanceConfig: appearanceConfig) { error in
                                                                                        reject("error", stringFromOnboardingError(error), error)
                                                                                }
                                
                                     onboardingCommands.authenticate { result in
                                              switch result {
                                                  case .success(let response):
                                                    let resposeDict: [String: String] = ["authentication_id": response.content as? String ?? ""]
                                                    let encoder = JSONEncoder()
                                                    if let jsonData = try? encoder.encode(resposeDict) {
                                                        if let jsonString = String(data: jsonData, encoding: .utf8) {
                                                            resolve(jsonString)
                                                        } else {
                                                            resolve("")
                                                        }
                                                    }
                                                  case .failure(let error):
                                                       reject("error", stringFromOnboardingError(error), error)
                                                  case .cancel:
                                                       reject("error", "cancel", nil)
                                                  case .dismissButton:
                                                       reject("error", "dismissButton", nil)
                                                  @unknown default:
                                                       reject("error", "unknown", nil)
                                                  }
                                          }
                                   }
                                 } else {
                                     reject("error", "camera permissions not granted", nil)
                               }
                         }
          }
    
    func isValidEmail(email: String) -> Bool {
        let emailRegEx = "[A-Z0-9a-z._%+-]+@[A-Za-z0-9.-]+\\.[A-Za-z]{2,64}"
        let emailPred = NSPredicate(format: "SELF MATCHES %@", emailRegEx)
        return emailPred.evaluate(with: email)
    }
    
    @objc(logInWithSandbox:email:firstName:lastName:resolve:reject:)
    func logInWithSandbox(sandboxToken: String,
                          email: String,
                          firstName: String? = "",
                          lastName: String? = "",
                          resolve: @escaping RCTPromiseResolveBlock,
                          reject: @escaping RCTPromiseRejectBlock) {
        
        if isValidEmail(email: email) {
            let sandboxManager = SandboxManager(sandboxToken: sandboxToken)
            sandboxManager.getUser(email: email) { resultGetUser in
                if case .success = resultGetUser {
                    sandboxManager.getUserToken(email: email) { resultGetUserToken in
                        switch resultGetUserToken {
                        case .success(let userToken):
                            resolve(userToken)
                        case .failure(let error):
                            reject("error", stringFromAuthenticationError(error), error)
                        }
                    }
                } else {
                    let userInfo = UserInfo(email: email,
                                            firstName: firstName!,
                                            lastName: lastName!)
                    sandboxManager.createUser(userInfo: userInfo) { resultCreateUser in
                        switch resultCreateUser {
                        case .success:
                            sandboxManager.getUserToken(email: email) { resultGetUserToken in
                                switch resultGetUserToken {
                                case .success(let userToken):
                                    resolve(userToken)
                                    //completion(.success(userToken))
                                case .failure(let error):
                                    reject("error", stringFromAuthenticationError(error), error)
                                }
                            }
                        case .failure(let error):
                            reject("error", stringFromAuthenticationError(error), error)
                        }
                    }
                }
            }
        } else {
            reject("error", "invalid email", nil)
        }
    }
}

