//  TransformationTools.swift
//  ALiCE Biometrics
//  Copyright © 2020+ ALiCE Biometrics. All rights reserved.


import Foundation
import AliceOnboarding


func getConfigDictionary(nsDictionaryConfig: NSDictionary) -> Dictionary<String, Any> {
       // NSDictionary to [String:Any] is not working properly

       var config = Dictionary<String, Any>()

       let stages = nsDictionaryConfig.object(forKey: "stages")
       let localization = nsDictionaryConfig.object(forKey: "localization")
       let appearance = nsDictionaryConfig.object(forKey: "appearance")
       let componentName = nsDictionaryConfig.object(forKey: "componentName")
       let componentVersion = nsDictionaryConfig.object(forKey: "componentVersion")


       if ((stages) != nil) {
           config["stages"] = stages as! [[String: Any]]
       }
       if ((localization) != nil) {
           config["localization"] = localization as! [String: Any]
       }
       if ((appearance) != nil) {
           let primaryColor = (appearance as AnyObject).object(forKey: "primaryColor")
           let secondaryColor = (appearance as AnyObject).object(forKey: "secondaryColor")
           let uncheckedItemsColor = (appearance as AnyObject).object(forKey: "uncheckedItemsColor")
           let fontRegular = (appearance as AnyObject).object(forKey: "fontRegular")
           let fontBold = (appearance as AnyObject).object(forKey: "fontBold")

           var appearanceDict = Dictionary<String, Any>()

           if ((primaryColor) != nil) {
               appearanceDict["primaryColor"] = primaryColor as! [String: Double]
           }
           if ((secondaryColor) != nil) {
               appearanceDict["secondaryColor"] = secondaryColor as! [String: Double]
           }
           if ((uncheckedItemsColor) != nil) {
               appearanceDict["uncheckedItemsColor"] = uncheckedItemsColor as! [String: Double]
           }
           if ((fontRegular) != nil) {
               appearanceDict["fontRegular"] = fontRegular as! String
           }
           if ((fontBold) != nil) {
               appearanceDict["fontBold"] = fontBold as! String
           }
           config["appearance"] =  appearanceDict
       }
      if ((componentName) != nil) {
        config["componentName"] = componentName as! String
      }
      if ((componentVersion) != nil) {
         config["componentVersion"] = componentVersion as! String
      }
      return config
}

func stringFromOnboardingError(_ onboardingError: OnboardingError) -> String {
    switch onboardingError {
    case .invalidUserTokenError(let statusCode, let message):
        return "\(statusCode): Invalid User Token \(message)"
    case .clientError(let statusCode, let message):
        return "\(statusCode): Client Error: \(message)"
    case .serverError(let statusCode, let message):
        return "\(statusCode): Server Error: \(message)"
    case .insufficientFaces:
        return "Insufficient Faces"
    case .invalidIssuingCountryError:
        return "Invalid Issuing Country Error"
    case .connectionError:
        return "Connection Error"
    case .encodingError:
        return "Encoding Error"
    case .decodingError:
        return "Decoding Error"
    case .unknownError:
        return "Unknown Error"
    case .languageNotFoundInBundle:
        return "Language not found in bundle"
    }
}


func stringFromAuthenticationError(_ authenticationError: AuthenticationError) -> String {
    switch authenticationError {
    case .invalidSandboxToken(let statusCode, let message):
        return "\(statusCode): Invalid Sandbox Token \(message)"
    case .userNotFound:
        return "User not Found Error"
    case .clientError(let statusCode, let message):
        return "\(statusCode): \(message)"
    case .serverError(let statusCode, let message):
        return "\(statusCode): \(message)"
    case .connectionError:
        return "Connection Error"
    case .encodingError:
        return "Encoding Error"
    case .unknownError:
        return "Unknown Error"
    case .invalidEmail:
        return "Invalid email"
    case .userAlreadyExist:
        return "User Already Exist"
    }
}
